<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
namespace Montonio\Hirepurchase\Gateway\Payments\Command;

/**
 * @inheritdoc
 */
class DataCommand implements \Magento\Payment\Gateway\CommandInterface
{
    /**
     *
     * @param \Montonio\Hirepurchase\Gateway\Payments\Config\Config $config
     * @param \Magento\Framework\Stdlib\DateTime\TimezoneInterface $timezoneInterface
     * @param \Magento\Framework\UrlInterface $urlInterface
     * @param \Magento\Framework\Locale\Resolver $localeResolver
     */
    public function __construct(
        \Montonio\Hirepurchase\Gateway\Payments\Config\Config $config,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $timezoneInterface,
        \Magento\Framework\UrlInterface $urlInterface,
        \Magento\Framework\Locale\Resolver $localeResolver
    ) {
            $this->config = $config;
            $this->timezoneInterface = $timezoneInterface;
            $this->urlInterface = $urlInterface;
            $this->localeResolver = $localeResolver;
    }

    /**
     * return payment request data
     *
     * @param array $commandSubject
     * @return string|Command\ResultInterface
     */
    public function execute(array $commandSubject)
    {
        if (!isset($commandSubject['order']) || !isset($commandSubject['method'])
        ) {
            throw new \InvalidArgumentException(__('Verify  payment request data $order, $method'));
        }
        $redirectUrl = $this->buildRequestData($commandSubject['order'], $commandSubject['method']);
        return $redirectUrl;
    }

    protected function buildRequestData($order, $method)
    {

        $storeId = $order->getStoreId();
        $environment = $this->config->getValue('environment', $storeId);

        $data = [];
        /* system data */
        $data['exp'] = $this->timezoneInterface->date()->format('U') + (60 * 10);
        $data['access_key'] = $this->config->getValue($environment.'_public_key', $storeId);

        $returnUrl = $this->config->getValue('return_url', $storeId);
        $hookUrl = $this->config->getValue('return_hook_url', $storeId);

        $data['merchant_return_url'] = $this->urlInterface->getUrl($returnUrl);
        $data['merchant_notification_url'] = $this->urlInterface->getUrl($hookUrl);

        //if grouped we don't send this so we get payment choices on gateway
        $isGrouped = $this->config->getValue('grouped');
        if (!$isGrouped) {
            $data['preselected_aspsp'] = $method;
        }

        $storeLocale = $this->localeResolver->getLocale();

        //montonio only supports following locales
        $locales = [
            'en_US' => 'en_US',
            'et_EE' => 'et',
            'lt_LT' => 'lt',
            'ru_RU' => 'ru'
        ];

        $localeCode = (isset($locales[$storeLocale])) ? $locales[$storeLocale] : $storeLocale;

        $data['preselected_locale'] = $localeCode;

        /*order data*/
        $data['amount'] = (float)sprintf("%.2f", $order->getGrandTotal());
        $data['currency'] = $order->getOrderCurrency()->toString();
        $data['merchant_reference'] = $order->getIncrementId();
        $data['payment_information_unstructured'] = __('Payment for order %1', $order->getIncrementId());
        //$data['payment_information_structured'] = '';

        if ($order->getCustomerEmail()) {
            $data['checkout_email'] = $order->getCustomerEmail();
        }

        if ($order->getBillingAddress()->getFirstname()) {
            $data['checkout_first_name'] = $order->getBillingAddress()->getFirstname();
        }

        if ($order->getBillingAddress()->getLastname()) {
            $data['checkout_last_name'] = $order->getBillingAddress()->getLastname();
        }

        if ($order->getTelephone()) {
            $data['checkout_phone_number'] =  $order->getTelephone();
        }

        $privateKey = $this->config->getValue($environment.'_private_key', $storeId);
        $applicationUrl= $this->config->getValue($environment.'_application_url', $storeId) . '/?payment_token=';
        $jwtToken = $this->config->generateSignature($data, $privateKey);

        $data = $applicationUrl . $jwtToken;

        return $data;
    }
}
